package scripts.migration.from4_4To4_5;

import scripts.migration.from4_4To4_5.form.ExtraFormTasks;
import scripts.migration.from4_4To4_5.general.ChangeReferencesToExtends;
import scripts.migration.from4_4To4_5.general.PostMigrationProcessing;
import scripts.migration.from4_4To4_5.general.RenameDialogsAndChangeInDialogsMap;
import scripts.migration.from4_4To4_5.general.TemplatesAndParagraphsToPagesAndComponents;
import scripts.migration.from4_4To4_5.general.PathChangesImpactingID;
import scripts.migration.from4_4To4_5.general.TransformExtends;
import scripts.migration.from4_4To4_5.general.UpdateMetaDataTemplateId;
import scripts.migration.from4_4To4_5.general.UpdateWebsitePrimaryType;
import scripts.migration.from4_4To4_5.general.TransformSimpleTemplates;
import scripts.migration.from4_4To4_5.pur.PURExtraTasks;
import scripts.migration.from4_4To4_5.resources.ResourcesExtraTasks;
import scripts.migration.from4_4To4_5.stk.RemoveTemplateToPrototypeRedundancy;
import scripts.migration.from4_4To4_5.stk.PreProcessSTKTemplates;
import scripts.migration.from4_4To4_5.stk.TransformSTKWebsites;
import scripts.migration.from4_4To4_5.stk.TransformSTKTemplates;
import scripts.migration.from4_4To4_5.stk.RemoveClassFromSTKSiteThemes;
import scripts.migration.from4_4To4_5.stk.ExtraSTKTasks;
import scripts.migration.from4_4To4_5.util.MigrationUtil;
import scripts.migration.from4_4To4_5.util.BuildDialogIdMap;
import scripts.migration.from4_4To4_5.util.ExportConfiguration;


// Defining the modules to operate on
// Modules for step 2
Collection modulesWithPagesAndComponents        = MigrationUtil.filterUninstalledModules(["commenting","public-user-registration","standard-templating-kit", "form",  "categorization","rssaggregator","forum", "dms"])
// Modules for step 3
Collection modulesRelevantForIDCreation         = MigrationUtil.filterUninstalledModules(["commenting","public-user-registration","standard-templating-kit", "form", "categorization","rssaggregator", "resources", "forum", "dms"])
// Modules for step 14
Collection modulesWithPossibleDialogReferences  = MigrationUtil.filterUninstalledModules(["commenting", "standard-templating-kit", "form", "categorization","rssaggregator", "resources","forum"])
// Modules for step 7
Collection stkModulesToPreprocess =  MigrationUtil.filterUninstalledModules(["standard-templating-kit"])
// Site definitions for step 7
Collection siteDefToPreprocess = ["/modules/standard-templating-kit/config/site", "/modules/extended-templating-kit/config/sites/default" ]

// Module for step 12
Collection transformSimpleTemplatesInModules    = MigrationUtil.filterUninstalledModules(["commenting","public-user-registration","form",  "categorization","rssaggregator", "resources","forum", "dms"])

// Websites for steps 5 - 6 - 8
// Defining the websites to operate on
Collection websites = MigrationUtil.filterNotExistingWebSites(["/demo-project", "/demo-features"])



/*******************************
 * Start of complete migration *
 *******************************/


/***** Start: Impacts and creation of dialogIdMap, pagesIdMap, componentsIdMap *****/
// 1. Change paths
pathChanges = new PathChangesImpactingID(ctx: ctx)
pathChanges.doChangePathes()

// 2. Move templates and paragraphs on modules containing such.
// Execute only on modules providing templating functionnality
// Attention: - mandatory step for ID creations!
templatesAndParagraphsToPagesAndComponents = new TemplatesAndParagraphsToPagesAndComponents(ctx: ctx)
templatesAndParagraphsToPagesAndComponents.doModules(modulesWithPagesAndComponents)

// 3. Initialization of dialogIdMap, pagesIdMap, componentsIdMap
Map dialogsIdMap = new BuildDialogIdMap(ctx: ctx).buildDialogIdMap(modulesRelevantForIDCreation)
Map pagesIdMap = MigrationUtil.buildPagesIdMapFromListOfModule(modulesRelevantForIDCreation, ctx)
Map componentsIdMap = MigrationUtil.buildComponentsIdMapFromListOfModule(modulesRelevantForIDCreation, ctx)

// 4. Renaming dialogs and changing within the dialogs map to the new value.
renameDialogsAndChangeInDialogsMap = new RenameDialogsAndChangeInDialogsMap(ctx:ctx, dialogsIdMap:dialogsIdMap)
renameDialogsAndChangeInDialogsMap.doRenameDialogs(RenameDialogsAndChangeInDialogsMap.dialogsToRename())
/***** End: Impacts and creation of dialogIdMap, pagesIdMap, componentsIdMap *****/



/***** Start: Migration of website content *****/
// 5. Updating website's primary type to pages&components&areas
updateWebsitePrimaryType = new UpdateWebsitePrimaryType(ctx: ctx)
updateWebsitePrimaryType.doWebsites(websites)
updateWebsitePrimaryType.log()

// 6. Updating meta data to new templateId
updateMetaData = new UpdateMetaDataTemplateId(ctx: ctx, pagesIdMap:pagesIdMap, componentsIdMap:componentsIdMap)
updateMetaData.doWebsites(websites)
updateMetaData.log()
// End Updating meta data to new templateId
/***** End: Migration of website content stop *****/



/***** Start: STK Migration *****/
// 7. Prepare STK templates
preProcessSTKTemplates = new PreProcessSTKTemplates(ctx: ctx)
preProcessSTKTemplates.doPreProcess(stkModulesToPreprocess, siteDefToPreprocess)

// 8. Transform STK web-sites
transformSTKWebsites = new TransformSTKWebsites(ctx: ctx)
transformSTKWebsites.doWebsites(websites);

// 9. Migration of STK module
transformSTKTemplates = new TransformSTKTemplates(ctx: ctx, componentsIdMap:componentsIdMap, pagesIdMap:pagesIdMap, dialogsIdMap:dialogsIdMap)
transformSTKTemplates.migrateSTK()

// 10. Remove STK's area redundancy
remove = new RemoveTemplateToPrototypeRedundancy(ctx: ctx)
remove.doSTK()
// remove class from themes found in the site definitions
removeClassFromSTKSiteThemes = new RemoveClassFromSTKSiteThemes()
removeClassFromSTKSiteThemes.convertSites()
removeClassFromSTKSiteThemes.log()
//
//
// 11. Transform Dialogs&Components&Pages with Extends
transformExtends = new TransformExtends(ctx:ctx)
Map stkComparisonMap = transformExtends.prepareAndCreateSTKComparisonMap(ctx)
transformExtends.executeComparison(stkComparisonMap, "standard-templating-kit/", "info.magnolia.module.templatingkit.messages")
transformExtends.deletedI18nMessagesLog()

// 11.1 Call extra task
stkExtraTask = new ExtraSTKTasks(ctx:ctx)
stkExtraTask.execute()
/***** End: STK Migration *****/



/***** Start: Modules Migration *****/
// 12. Transform templates simple templates (non STK templates)
transformTemplates = new TransformSimpleTemplates(ctx: ctx, dialogsIdMap:dialogsIdMap)
transformTemplates.doModules(transformSimpleTemplatesInModules)
transformTemplates.log()

// 13. Extra tasks
// Form module's extra tasks
extraFormTask = new ExtraFormTasks(ctx: ctx, componentsIdMap:componentsIdMap)
extraFormTask.execute()

// Resources modules's extra tasks
resourcesExtraTasks = new ResourcesExtraTasks(ctx: ctx)
resourcesExtraTasks.execute()

// PUR modules's extra tasks
purExtraTasks = new PURExtraTasks(ctx: ctx, componentsIdMap:componentsIdMap)
purExtraTasks.execute()
purExtraTasks.updateGroupFiedsTemplateOnWebsites(websites)
purExtraTasks.updateOldProperties()
/***** End: None STK Modules Migration *****/



/***** Start: All post module migration changes *****/
// 14. ChangeReferencesToExtends
// Replace remaining References with Extends
changeReferencesToExtends = new ChangeReferencesToExtends(ctx:ctx)
changeReferencesToExtends.doModules(modulesWithPossibleDialogReferences)

// 15 Post Migration
//Adding and changing values, after the complete Migration is done
postMigrationProcessing = new PostMigrationProcessing(ctx:ctx)
postMigrationProcessing.executePropertyChanges()
postMigrationProcessing.executeImportBootstraps()
postMigrationProcessing.executeDeleteNodes()
postMigrationProcessing.executeDeleteProperties()
/***** End: All post module migration changes *****/


/*****************************
 * End of complete migration *
 *****************************/
println "Migration done!"


/**
 * Possible triggering of exporting all changes:
 * These are two examples, on how you could trigger an export of any module configuration.
 */
///** Start exporting module configurations **/
//exportConfiguration = new ExportConfiguration(ctx: ctx)
////Example 1: just exporting the the "dialogs" and "templates" folder within the STK module
//exportConfiguration.exportModule(["standard-templating-kit"], "/Users/cringele/Documents/temp_stuff/migration/export_test", ["dialogs", "templates"])
////Example 2: exporting all sub folders of the STK & Form module.
//exportConfiguration.exportModule(["standard-templating-kit", "form"], "/Users/cringele/Documents/temp_stuff/migration/export_test", null)

////Example 3: exporting wesites
//exportConfiguration.exportWebsite(["demo-project","demo-features"], "/Users/erichechinger/Dev/Workspaces/Magnolia_STK_2_0_Bootstrap/Migration/generated", null)


///** End exporting module configurations **/

