package scripts.migration.from4_4To4_5.general;

import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.context.Context;
import info.magnolia.context.MgnlContext;
import org.apache.commons.lang.StringUtils;

import scripts.migration.from4_4To4_5.util.MigrationUtil;

 /**
  * Dialogs, Pages and Components are now referenced by their id (moduleName:pathToItem) and not anymore by their name.
  * In this script all path changes are done which could have an impact on creation of the id map used in:
  * <code>dialogsIDsMap = convertToNewDialogPath.initDialogPathMapFromListOfModule(moduleList)</code>
  * <code>pagesIDsMap = MigrationUtil.buildPagesIdMapFromListOfModule(moduleList,ctx)</code>
  * <code>componentsIDsMap = MigrationUtil.buildComponentsIdMapFromListOfModule(moduleList,ctx) </code>
  * So the pathes are changed before any id is created and later used.
  **/

class PathChangesImpactingID{

    Context ctx
    HierarchyManager hm

    /**
     * Initial method to call to start the process.
     * Right place to ad more logic if needed.
     */
    def doChangePathes(){
        hm = MgnlContext.getHierarchyManager('config');

        renameNodes(mapOfNodesToRename());
    }

    /**
     * Renames all nodes defined in the map <code>mapOfNodesToRename()</code>.
     */
    def renameNodes(Map nodesToRename){

        nodesToRename.each{ String pathOfNode, String newNodeName ->
            if(hm.isExist(pathOfNode)){
                MigrationUtil.renameNode(hm.getContent(pathOfNode), newNodeName)
            }
        }
    }

    /**
     * Key is path to node, value new node name.
     * @return Map of all paths to change
     */
    def static Map mapOfNodesToRename(){
        def nodesToRename = [
            "/modules/standard-templating-kit/dialogs/paragraphs" : "components"
          ]
        return nodesToRename;
    }


}
