package scripts.migration.from4_4To4_5.stk;

import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.context.Context;
import scripts.migration.from4_4To4_5.util.MigrationUtil;
import org.apache.commons.lang.StringUtils;
import javax.jcr.Property
import javax.jcr.Session;
import javax.jcr.Node;


import info.magnolia.context.MgnlContext;
import info.magnolia.jcr.util.NodeUtil;
/**
 * The goal of this step is to do the extra tasks of STK.
 * All the tasks that are not directly linked to the main migration.
 * An extra task could remove inconsistency, like for example if a dialog is not properly named and all other stuff like that.
 *
 * The goal of this task is also to reduce the complexity of the other steps. We often remarked that some special cases have to be handled in the migration of STK.
 * This could be to best place for those kind of things.
 *
 * This step has to be executed at the very end of the process.
 *
 */
class ExtraSTKTasks {

  Context ctx
  HierarchyManager hm

  def execute(){
      hm = ctx.getHierarchyManager('config');

      //Add templateScript/title property for stkHome and stkSection
      def stkHome = hm.getContent('/modules/standard-templating-kit/templates/pages/stkHome/areas/main/areas/content');
      stkHome.createNodeData('templateScript','/templating-kit/pages/section/contentArea.ftl');
      stkHome.createNodeData('title','templates.stkHome.areas.main.content.title');
      def stkSection = hm.getContent('/modules/standard-templating-kit/templates/pages/stkSection/areas/main/areas/content');
      stkHome.createNodeData('templateScript','/templating-kit/pages/section/contentArea.ftl');
      stkHome.createNodeData('title','templates.stkSection.areas.main.content.title');

      hm.delete("/modules/standard-templating-kit/dialogs/generic/teasers/tabLinkList");

      Map newParameters = [
          "headingLevel" : "h3"
          ]
      def stkTeaserLinkList = hm.getContent('/modules/standard-templating-kit/templates/components/teasers/stkTeaser/areas/linkList');
      MigrationUtil.createParameters(stkTeaserLinkList,newParameters);

      Session session = MgnlContext.getJCRSession("website");
      NodeUtil.getNodes(session.rootNode, "mgnl:page").each{processLinkList(it)}
      session.save()

      //TODO: footer component need to be removed.


   }

  /**
   * move linkListTitle values and remove hasLinkArea properties.
   */
  def processLinkList(Node node){
      println (node.path)
      if(node.hasProperty("hasLinkList")){
          node.getProperty("hasLinkList").remove()
      }
      if(node.hasProperty("linkListTitle")){
          Property property = node.getProperty("linkListTitle")
          Node area;
          if(!node.hasNode("linkList"))
              area = node.addNode("linkList", "mgnl:area")
          else
              area = node.getNode("linkList")
          area.setProperty("title", property.getString())
          property.remove()
      }
      node.getNodes().each{processLinkList(it)}
  }
}
