package scripts.migration.from4_4To4_5.stk;

import java.util.Collection;
import java.util.Map;

import javax.swing.plaf.MenuItemUI;

import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.cms.core.ItemType;
import info.magnolia.cms.util.ContentUtil
import info.magnolia.cms.util.NodeDataUtil;
import info.magnolia.cms.util.PathUtil;
import info.magnolia.context.Context;
import info.magnolia.context.MgnlContext;

import org.apache.commons.lang.StringUtils;

import scripts.migration.from4_4To4_5.util.MigrationUtil;

import info.magnolia.rendering.template.AreaDefinition;

class TransformSTKTemplates {

  Context ctx
  HierarchyManager hm

  // this map must contain only ids of the components
  Map componentsIdMap = [:]
  Map pagesIdMap = [:]
  Map dialogsIdMap = [:]
  //for log
  Collection componentNotChanged = []
  Collection templateAvailibilityNotChanged = []
  Collection dialogNotChanged = []
  Collection subParagraphsNotInList = []

  //List of optional area
  Collection optionalArea = ["stage", "opener"]

  //Map of area:type
  Map areaTypeForSTK = [
    "htmlHeader":AreaDefinition.TYPE_NO_COMPONENT,
    "branding":AreaDefinition.TYPE_NO_COMPONENT,
    "logo":AreaDefinition.TYPE_NO_COMPONENT,
    "search":AreaDefinition.TYPE_NO_COMPONENT,
    "metaNavigation":AreaDefinition.TYPE_LIST,
    "sectionHeader":AreaDefinition.TYPE_NO_COMPONENT,
    "stage":AreaDefinition.TYPE_SINGLE,
    "platformArea":AreaDefinition.TYPE_LIST,
    "breadcrumb":AreaDefinition.TYPE_NO_COMPONENT,
    "mainArea":AreaDefinition.TYPE_NO_COMPONENT,
    "content":AreaDefinition.TYPE_LIST,
    "intro":AreaDefinition.TYPE_NO_COMPONENT,
    "infoBlock":AreaDefinition.TYPE_NO_COMPONENT,
    "comments":AreaDefinition.TYPE_SINGLE,
    "extrasArea":AreaDefinition.TYPE_NO_COMPONENT,
    "extrasArea1":AreaDefinition.TYPE_LIST,
    "extrasArea2":AreaDefinition.TYPE_LIST,
    "module":AreaDefinition.TYPE_SINGLE,
    "promosArea":AreaDefinition.TYPE_LIST,
    "baseArea":AreaDefinition.TYPE_LIST,
    "footer":AreaDefinition.TYPE_LIST,
    "opener":AreaDefinition.TYPE_SINGLE
  ]



  /////////////////////////////////////////
  /// create an 'areas' node reflecting the Map<String name, AreaConfiguration>.
  //FIXME SCRUM-433 - use the method form MigrationUtil
  def createAreasNode(Content content)
  {
    if(!content.hasContent("areas"))
    {
      def area = content.createContent("areas", ItemType.CONTENTNODE);
      content.save();
    }
  }



  /**
   * Transform a paragraphs node into a availableComponents.
   * Update the paragraphs ids.
   */
  def modifyParagraph(Content content)
  {
    if(content.hasContent("paragraphs"))
    {
      def para = content.getContent("paragraphs")
      para.getChildren().each{Content child -> updateAvailableComponent(child)}
      para.save();
      ContentUtil.rename(para,"availableComponents")
      return true
    }
    return false
  }

  /**
   * Create the type property for an Area.
   * If the area has components, type is 'list' else is 'noComponent'.
   * For STK areas we set the type found in the map.
   */
  def createTypeProperty(Content area, boolean hasComponents)
  {
    if(areaTypeForSTK.containsKey(area.getName())){
      area.setNodeData("type", areaTypeForSTK[area.getName()])
    }
    else{
      if(hasComponents){
        area.setNodeData("type",AreaDefinition.TYPE_LIST)
      }
      else{
        area.setNodeData("type",AreaDefinition.TYPE_NO_COMPONENT)
      }
    }
    area.save()
  }

  /**
   * Update a component
   *
   */
  def updateAvailableComponent(Content component){
    if(componentsIdMap.containsKey(component.getName())){
      def id = componentsIdMap.get(component.getName());
      component.setNodeData("id", id)
      if(component.hasNodeData("name")){
        component.deleteNodeData("name");
      }
      component.save();
    }
    else{
      componentNotChanged.add("Id not found for [ "+component.getName()+" ]. Please update this component: "+component.getHandle())
    }
  }

  /**
   * Update a template avaibility
   *
   */
  def updateTemplateAvailibility(Content content)
  {
    if(pagesIdMap.containsKey(content.getName())){
      def id = pagesIdMap.get(content.getName())
      content.setNodeData("id", id)
      if(content.hasNodeData("name")){
        content.deleteNodeData("name");
      }
      content.save();
    }
    else{
      templateAvailibilityNotChanged.add("Id not found for template [ "+content.getName()+" ]. Please update this template availibility: "+content.getHandle())
    }
  }

  /**
   * Creates the available components content node for the paragraph.
   * This is needed for the special case of paragraphs having one ore more subParagraphs. The list needs to be transformed into
   * the following structure:
   * paragraph-with-subparagraphs
   * |_ areas
   *    |_ collection-name (looked up in corresponding .ftl)
   *       |_ availableComponents
   *          |_ item1
   *             |_ id   path
   *          |_ item2
   *             |_ id   path
   *
   * This function creates the availableContent content node and all its children.
   *
   *
   * @param parent Node to operate on
   * @param subParagraphList Comma separated list (String) containing the names of the subParagraphs
   *
   * FIXME SCRUM-433 - use MigrationUtil. createAvailableComponents
   */
  def createAvailableComponents(Content parent, String subParagraphList)
  {
    def componentNames = StringUtils.split(subParagraphList, ',')

    //create availableComponents content node if it doesn't exist already
    if (!parent.hasContent("availableComponents")) {
      parent.createContent("availableComponents", ItemType.CONTENTNODE)
      parent.save()
    }

    def availableComponents = parent.getContent("availableComponents")

    // create available components
    for (componentName in componentNames) {
      if(componentsIdMap.containsKey(componentName)){
        def id = componentsIdMap.get(componentName)

        //create component
        def component = availableComponents.createContent(componentName, ItemType.CONTENTNODE)
        availableComponents.save()
        //set its id
        component.setNodeData("id", id)
      }
      else{
        componentNotChanged.add("Id not found for [ "+componentName+" ].")
      }
    }
    parent.save()
  }

  /**
   * Update either the properties name or value or both.
   * Handle some special cases.
   *
   * @param content
   * @return
   */
  def updateTemplateProperties(Content content)
  {
    MigrationUtil.updateTemplatePropertiesGeneric(content, dialogsIdMap,dialogNotChanged)
    //for sectionHeader area
    //add a dialog before STK 2.0 it was hardcoded in templates/section/sectionHeader.ftl
    if(content.getName().equalsIgnoreCase("sectionHeader")){
      content.setNodeData("dialog", dialogsIdMap["stkSectionBanner"])
    }

    //for opener area
    //add a template script,  before STK 2.0 it was hardcoded in mainArea.ftl
    if(content.getName().equalsIgnoreCase("opener")){
      content.setNodeData("templateScript", "/templating-kit/pages/section/opener.ftl")
      content.setNodeData("title", "areas.templates.main.opener.title")
      content.setNodeData("description", "areas.templates.main.opener.description")
    }

    //for footer area
    //remove paragraph property and add a dialog property with value is the dialog of the removed paragraph
    if(content.getName().equalsIgnoreCase("footer") && content.hasNodeData("paragraph")){
      def footerDialog = content.getNodeData("paragraph").getString()
      content.setNodeData("dialog", dialogsIdMap[footerDialog])
      content.deleteNodeData("paragraph")
    }

    //add a the new 'module'-area a script which was not existing yet. Module area code was placed in the extras area script.
    if(content.getName().equalsIgnoreCase("module")){
      content.setNodeData("templateScript", "/templating-kit/pages/global/moduleArea.ftl")
    }

    //The breadcrumbArea has a special model
    if(content.getName().equalsIgnoreCase("breadcrumb")){
      content.setNodeData("modelClass", "info.magnolia.module.templatingkit.templates.areas.BreadcrumbAreaModel")
    }

    //The sectionHeaderArea has a special model
    if(content.getName().equalsIgnoreCase("sectionHeader")){
      content.setNodeData("modelClass", "info.magnolia.module.templatingkit.templates.areas.SectionHeaderAreaModel")
    }

    //The footerArea has a special model
    if(content.getName().equalsIgnoreCase("footer")){
      content.setNodeData("modelClass", "info.magnolia.module.templatingkit.templates.areas.FooterAreaModel")
    }

    content.save()
  }

  // Areas extending info.magnolia.module.templatingkit.templates.Area need a property class defining their java class
  def updateClassProperty(Content content)
  {
    //map containing areaName / class
    def areasClass = [:];
    areasClass.put("mainArea", "info.magnolia.module.templatingkit.templates.MainArea");
    areasClass.put("extrasArea", "info.magnolia.module.templatingkit.templates.ExtrasArea");
    areasClass.put("intro", "info.magnolia.module.templatingkit.templates.MainAreaIntro");
    areasClass.put("comments", "info.magnolia.module.templatingkit.templates.CommentsArea");

    if(!content.hasNodeData("class") && areasClass.containsKey(content.getName()))
    {
      content.setNodeData("class", areasClass[content.getName()]);
      content.save();
    }
  }

  def updateAutoGeneratedParagraph(Content area){
    //Main area definition contained always only one auto generated paragraph, and is declared as autoGeneratedParagraph. By the pre-process scipt it was moved into the new 'content' sub area of main area.
    if(area.hasContent("autoGeneratedParagraph")){
      String componentName = "singleton"
      // get the old configuration information
      def oldAutoGenConfig = area.getContent("autoGeneratedParagraph")
      transformAutoGenParagraph(area, oldAutoGenConfig, componentName)

      //we assume that in this case the area is type of single
      area.setNodeData("type",AreaDefinition.TYPE_SINGLE)
      MigrationUtil.addGenericTemplateScript(area,"single")
    }
    //Extras area can contain a list of auto generated paragraphs within autoGeneratedParagraphs. The children are the equivalent to a single 'autoGeneratedParagraph' in main area.
    if(area.hasContent("autoGeneratedParagraphs")){
      // get the old configuration information
      String componentName = "extrasItem"
      int i = 0
      Content oldAutoGenParagraphs = area.getContent("autoGeneratedParagraphs")
      oldAutoGenParagraphs.getChildren().each{ Content oldAutoGenConfig ->
        transformAutoGenParagraph(area, oldAutoGenConfig, componentName+i)
        i = i+1
      }
      oldAutoGenParagraphs.delete()
    }
    area.save()
  }

  //FIXME SCRUM-433 maybe we should add more comments (ironic)
  def private transformAutoGenParagraph(Content area, Content oldAutoGenConfig, String componentName) {
    String id = componentsIdMap.get(oldAutoGenConfig.getNodeData("name").getString())
    ///Get the defaultValues if existing. This is the base for the new auto generate component.
    Content defaultValues = MigrationUtil.getOrCreateContent(oldAutoGenConfig, "defaultValues", ItemType.CONTENTNODE)
    //The type the auto generated node should have. We assume its all components.
    defaultValues.setNodeData("nodeType", ItemType.COMPONENT)
    //Setting the templateId of the component.
    defaultValues.setNodeData("templateId", id)

    //create the generators configuration if not exisitng
    Content autoGenerationNode = MigrationUtil.getOrCreateContent(area, "autoGeneration", ItemType.CONTENTNODE)
    //Set generator class
    autoGenerationNode.setNodeData("generatorClass","info.magnolia.rendering.generator.CopyGenerator")
    //Create the content node if existing. This is the node represending the structure whcih should be auto generated.
    Content content = MigrationUtil.getOrCreateContent(autoGenerationNode, "content", ItemType.CONTENTNODE)
    //Move the defaultValues into the content node and rename them.
    ContentUtil.moveInSession(defaultValues, content.getHandle()+"/"+componentName)
    //delete the old configuration node
    oldAutoGenConfig.delete()
  }

  // Remove from the children the ones that do not have to be moved in an area, like:availableComponents, subTemplates, parameters, areas
  def removeIgnoredChildren(Collection children)
  {
    def shortChildrenList = children.findAll {
      !(it.getName() in [
        "availableComponents",
        "subTemplates",
        "parameters",
        "areas",
        "formProcessors",
        "autoGeneratedParagraph",
        "autoGeneratedParagraphs",
        "autoGeneration",
        "inheritance",
        "floating"
      ])
    }
    return shortChildrenList;
  }

  /**
   * Create the inheritance configuration for an area.
   *
   * @param area
   * @return
   */
  def createInheritance(Content area){
    Map areaWithInheritance = [
          "extrasArea" : ["enabled" : true, "components" : "filtered"],
          "promosArea" : ["enabled" : true, "components" : "filtered"],
          "footer" : ["enabled" : true, "components" : "all"],
          "metaNavigation" : ["enabled" : true, "components" : "all"]
        ]
    if(areaWithInheritance.containsKey(area.getName())){
      Content inheritanceNode = area.createContent("inheritance", ItemType.CONTENTNODE)
      Map inheritanceProperties = areaWithInheritance.get(area.getName())
      inheritanceProperties.each{ String propName, def propValue ->
        inheritanceNode.setNodeData(propName, propValue)
      }
      area.save()
    }
  }

  def createOptionalProperty(Content area){
    if(optionalArea.contains(area.getName())){
      area.setNodeData("optional", true)
      area.save()
    }
  }

  /**
   * Transform a STKArea to the new configuration (AreaDefinition)
   */
  def Content transformSTKArea(Content stkArea)
  {
    updateTemplateProperties(stkArea)
    updateClassProperty(stkArea)
    def hasComponents = modifyParagraph(stkArea)
    createTypeProperty(stkArea, hasComponents)
    updateAutoGeneratedParagraph(stkArea)
    createInheritance(stkArea)
    createOptionalProperty(stkArea)
    //call updateDeprecatedClass after createInheritance
    MigrationUtil.updateClassAndModelClass(stkArea, MigrationUtil.mapOfDeprecatedClassesToReplace())
    MigrationUtil.updateChangedTemplateScripts(stkArea, MigrationUtil.mapOfChangedFreemarkerScripts())
    MigrationUtil.updateBasePathInTemplateScriptProperty(stkArea)
    MigrationUtil.updateClassAndModelClass(stkArea, MigrationUtil.mapOfMovedClasses())

    //special case "comments"
    if(stkArea.getName() == "comments"){
      transformCommentsArea(stkArea)
    }

    if(stkArea.hasChildren()){
      def children = stkArea.getChildren();
      children = removeIgnoredChildren(children);

      // if area has subAreas, a areasMap is created
      if(children.size()>0){
        createAreasNode(stkArea);
      }
      // transform the subAreas and move them into the areasMap
      children.each {
        transformSTKArea(it);
        ContentUtil.moveInSession(it,PathUtil.createPath(stkArea.getHandle()+"/areas",it.getName()));
      }
    }
    stkArea.save();
    return stkArea;
  }

  /**
   * Here you find the list of area to transform to the new configuration.
   *
   * @param children
   * @return
   */
  def getSTKAreaToTransform(Collection children)
  {
    def areasToTransform = [
      "htmlHeader",
      "branding",
      "sectionHeader",
      "stage",
      "platformArea",
      "breadcrumb",
      "mainArea",
      "extrasArea",
      "promosArea",
      "baseArea",
      "footer",
      "meta"
    ];
    def shortChildrenList = children.findAll {
      it.getName() in areasToTransform
    }
    return shortChildrenList;
  }

  /**
   * Creates the new areas content node and initiates the creation of its children for a paragraph with
   * subParagraphs.
   * See @createAvailableComponents for further details on the structure.
   *
   * @param content Node to operate on
   * @param subParagraphList List with one or more subParagraphs that will be passed to another function
   *
   */
  def migrateParametersSubparagraphs(Content content, String subParagraphList)
  {
    def specialParagraphAreaNamesMap = [
          "stkExtrasBoxGroup":"teaserGroupItems",
          "stkExtrasHorizontalTabbed":"horizontalTabItems",
          "stkFAQ":"faqItems",
          "stkFooterLinkList":"linkList",
          "stkLinkList":"linkList",
          "stkStagePaging":"carouselItems",
          "stkTeaserCarousel":"carouselItems",
          "stkTeaserFingerTabbed":"fingerTabItems",
          "stkTeaserGroup":"teaserGroupItems",
          "stkTeaserHorizontalTabbed": "horizontalTabItems",
          "stkTeaserPaging":"carouselItems"
        ]

    def specialScriptsForAreaMap = [
          "fingerTabItems":"/templating-kit/components/teasers/fingerTabbedArea.ftl",
          "horizontalTabItems":"/templating-kit/components/teasers/horizontalTabbedArea.ftl",
          "faqItems":"/templating-kit/components/features/faqArea.ftl",
          "carouselItems":"/templating-kit/components/teasers/carouselArea.ftl",
          "linkList":"/templating-kit/components/links/linkListArea.ftl"
        ]

    // FIXME SCRUM 433 - use MigrationUtil#createArea
    // create the 'areas' node
    createAreasNode(content)
    // get the area name from the list and create the definition
    def areas = content.getContent("areas")

    if (specialParagraphAreaNamesMap.containsKey(content.getName())){
      def subarea = areas.createContent(specialParagraphAreaNamesMap.get(content.getName()), ItemType.CONTENTNODE)
      areas.save()

      // set type to 'list'
      subarea.setNodeData("type","list")

      // need a templateScript
      if(specialScriptsForAreaMap.containsKey(subarea.getName())){
        subarea.setNodeData("templateScript", specialScriptsForAreaMap.get(subarea.getName()))
      } else {
        MigrationUtil.addGenericTemplateScript(subarea, "list")
      }

      // create available components in this sub-area
      createAvailableComponents(subarea, subParagraphList)
    }
    else {
      subParagraphsNotInList.add("Could not find "+content.getName()+" in the special paragraphs - please verify!")
    }

    content.save()
  }


  /**
   * Main method migrating a template like stkHome or paragraph like stkTextImage.
   * At the end template and paragraph will be represented by the same object.
   *
   * @param template
   * @return
   */
  def migrateTemplateDefinition(Content template)
  {
    println "Start Migration of "+template.getName();
    updateTemplateProperties(template)
    modifyParagraph(template)
    MigrationUtil.updateClassAndModelClass(template, MigrationUtil.mapOfDeprecatedClassesToReplace())
    MigrationUtil.updateChangedTemplateScripts(template, MigrationUtil.mapOfChangedFreemarkerScripts())
    MigrationUtil.updateBasePathInTemplateScriptProperty(template)
    MigrationUtil.updateClassAndModelClass(template, MigrationUtil.mapOfMovedClasses())

    // list of STKAreas to move in the areas node
    def areasToTransform = getSTKAreaToTransform(template.getChildren());

    if(areasToTransform.size() > 0)
    {
      createAreasNode(template);

      areasToTransform.each {
        def area = transformSTKArea(it);

        if(area.getName()=="footer"){
          area = specialTransformFooterArea(area)
        }

        if(area != null){
          //move and rename the area (remove Area suffix)
          ContentUtil.moveInSession(area, PathUtil.createPath(template.getHandle()+"/areas", rename(area)));
        }
      }
    }
    createContentNavigationArea(template)
    if(template.hasContent("areas") && template.getContent("areas").hasContent("extras")){
      transformExtrasToSubAreas(template.getContent("areas").getContent("extras"))
    }
  }


  /**
   * Return the new name of an area.
   * If you need to rename an area, the logic has to be placed here.
   *
   * @param area node
   * @return
   */
  def static rename(Content area){
    def areaName = area.getName()
    if(StringUtils.contains(areaName,"Area")){
      return StringUtils.remove(areaName,"Area");
    }
    else return areaName;
  }

  /**
   * Migrate STK based paragraphs.
   * Get all the templates found in /components folder and run the migrateTemplateDefinition method.
   *
   */
  def migrateSTKParagraph(moduleName){
    hm = ctx.getHierarchyManager('config');
    def templateFolder = hm.getContent('/modules/'+moduleName+'/templates/components')
    def templatesList = []
    MigrationUtil.visitTemplates(templateFolder, templatesList)
    templatesList.each {
      //Handle special case where paragraph has a list of subparagraph. Now these subparagraphs become areas.
      if (it.hasContent("parameters")) {
        // looking for node data subParagraph or subParagraphs
        def subParagraphs = it.getContent("parameters").getNodeDataCollection("subParagraph*")
        if (subParagraphs.size() > 0) {
          migrateParametersSubparagraphs(it, subParagraphs.getAt(0).getValue().getString())
          subParagraphs.getAt(0).delete()
          it.save()
        }
      }
      // special case since the subParagraphList was hard-coded in the freemarker template
      else if ((it.getName()).equalsIgnoreCase("stkLinkList")) {
        migrateParametersSubparagraphs(it, "stkInternalLink,stkExternalLink,stkDownloadLink")
      }
      // after preparing the special cases we always want to migrate the definitions
      migrateTemplateDefinition(it)
      createNewAreaForComponent(it)
    }
    hm.save()
  }

  /**
   * Before STK 2.0, some pararaphs handled the rendering of content via a macro. This mechanism is now obsolete, the macro has to be replaced by an area.
   * This method create a new area wihin a component.
   *
   * @param component
   * @return
   */
  def createNewAreaForComponent(Content component){
    // This complex map represents the structure of the area to create
    def areaObject = [
          "areaName":"linkList",
          "type":AreaDefinition.TYPE_LIST,
          "enabled": true ,
          "templateScript":"/templating-kit/components/teasers/linkListArea.ftl",
          "availableComponents":[
            "stkInternalLink",
            "stkExternalLink",
            "stkDownloadLink"
          ]
        ]

    // this map should contain as key the name of a component and as value the structure of the area to create.
    // Here by chance, the 3 components create the same area, otherwise we should create a complexe map
    def mapOfPargraphsWithMacro = ["stkTeaser":areaObject,"stkExtrasInternalPage":areaObject,"stkExtrasPureLinkList":areaObject, "stkTeaserPureLinkList":areaObject]

    if(mapOfPargraphsWithMacro[component.getName()]!=null){
      createAreasNode(component)
      MigrationUtil.createArea(component.getContent("areas"), mapOfPargraphsWithMacro[component.getName()], componentsIdMap)
    }
  }

  /**
   * Migrate STK based templates.
   * Get all the templates found in /pages folder and run the migrateTemplateDefinition method.
   *
   * @return
   */
  def migrateSTKTemplate(moduleName)
  {
    hm = ctx.getHierarchyManager('config');
    def templateFolder = hm.getContent('/modules/'+moduleName+'/templates/pages')
    def templatesList = []
    MigrationUtil.visitTemplates(templateFolder, templatesList)
    templatesList.each{ migrateTemplateDefinition(it) }
    hm.save();
  }

  /**
   * Migrate the default site definitions provided by STK and ETK.
   *
   */
  def migrateSTKSiteDefinition()
  {
    hm = ctx.getHierarchyManager('config')
    String stkSiteDefinitionPath = "/modules/standard-templating-kit/config/site"
    String etkSiteDefinitionPath = "/modules/extended-templating-kit/config/sites/default"

    migrateSiteDefinitions([stkSiteDefinitionPath])
    //ETK && EE is installed
    if(hm.isExist(etkSiteDefinitionPath)){
      migrateSiteDefinitions([etkSiteDefinitionPath])
    }
  }

  /**
   * This method can be used to update project site definitions. A list of pathes to the site definitions can be passed.
   *
   * @param siteDefinitionPaths List containng pathes to all SiteDefinitions which should be updated.
   */
  def migrateSiteDefinitions(Collection siteDefinitionPaths){
    siteDefinitionPaths.each{ String siteDefinitionPath ->
      // update prototype
      hm = ctx.getHierarchyManager('config');
      if(hm.isExist(siteDefinitionPath)) {
        Content prototype = hm.getContent(siteDefinitionPath+'/templates/prototype');
        migrateTemplateDefinition(prototype)
        if(prototype.hasContent("navigation")){
          if(prototype.getContent("navigation").hasContent("horizontal")){
            MigrationUtil.replaceSubstringInProperty(prototype.getContent("navigation").getContent("horizontal"), "template", "/templating-kit/templates/", "/templating-kit/pages/" )
          }
          if(prototype.getContent("navigation").hasContent("vertical")){
            MigrationUtil.replaceSubstringInProperty(prototype.getContent("navigation").getContent("vertical")  , "template", "/templating-kit/templates/", "/templating-kit/pages/" )
          }
        }

        createLogoAndSearchInBranding(prototype)

        //update template availibilty
        if(hm.isExist(siteDefinitionPath+'/templates/availability/templates')){
          Content templateAvailibility = hm.getContent(siteDefinitionPath+'/templates/availability/templates');
          templateAvailibility.getChildren().each{ Content child -> updateTemplateAvailibility(child) }
        }
        hm.save();
      }
    }
  }

  /**
   * Main method to call. Running the migration of templates, paragraphs and prototype of STK.
   *
   */
  def migrateSTK()
  {
    resetLog()
    println "Start Migration of module: standard-templating-kit"
    migrateSTKTemplate("standard-templating-kit")
    migrateSTKParagraph("standard-templating-kit")
    migrateSTKSiteDefinition()
    log("standard-templating-kit")
  }


  /**
   * Main method to call if you want to migrate a module providing template STK based.
   *
   * @param moduleName the name of the module
   * @param siteDefList list of site def
   */
  def migrateModule(moduleName, siteDefList)
  {
    resetLog()
    println "Start Migration of module: "+moduleName
    migrateSTKTemplate(moduleName)
    migrateSTKParagraph(moduleName)
    migrateSiteDefinitions(siteDefList)
    log(moduleName)
  }

  /**
   * Main method to call if you want to migrate a list of modules providing template STK based.
   *
   * @param moduleName List of modules names
   */
  def migrateModules(moduleNamesList)
  {
    moduleNamesList.each{
      println "Start Migration of module: "+it
      resetLog()
      migrateSTKTemplate(it)
      migrateSTKParagraph(it)
      log(it)
    }
  }

  def log(moduleName){
    println " *** "
    println " *** Start log summary of template transformation in module: "+moduleName
    println " *** "
    println " ** Components Not Changed: "
    println      componentNotChanged
    println " ** Template availibility Not Changed: "
    println      templateAvailibilityNotChanged
    println " ** Dialogs Not Changed: "
    println      dialogNotChanged
    if (subParagraphsNotInList.size() > 0) {
      println " ** Need to update the list for the subparagraphs in: "
      println      subParagraphsNotInList
    }
    println " *** "
    println " *** End log summary of template transformation in module: "+moduleName
    println " *** "
  }

  def resetLog(){
    componentNotChanged = []
    dialogNotChanged = []
    templateAvailibilityNotChanged = []
    subParagraphsNotInList = []
  }

  /**
   *  Creating the logo and search area within the branding area.
   */
  def createLogoAndSearchInBranding(Content templateNode){
    if(templateNode.hasContent("areas") && templateNode.getContent("areas").hasContent("branding")){
      Content bandingNode = templateNode.getContent("areas").getContent("branding")
      Content brandingAreasNode = MigrationUtil.createAreasNode(bandingNode)
      MigrationUtil.createArea(brandingAreasNode, "logo", "/templating-kit/pages/global/logo.ftl", areaTypeForSTK.get("logo"), true, null, null, null, null, null)
      MigrationUtil.createArea(brandingAreasNode, "search", "/templating-kit/pages/global/search.ftl", areaTypeForSTK.get("search"), true, null, null, null, null, null)
    }
  }


  /**
   * Create the new area "contentNavigation". This was done in prior STK versions by a template include.
   * Now the logic is transferred to the area model, and there for the proper solution is an area with its model.
   */
  def createContentNavigationArea(Content templateNode){
    if(templateNode.hasContent("areas")
    && templateNode.getContent("areas").hasContent("main")
    && templateNode.getContent("areas").getContent("main").hasContent("areas")){

      def areasNode = templateNode.getContent("areas").getContent("main").getContent("areas")
      def templateName = templateNode.getName();

      //Adding the contentNavigation area to the prototype disabled, but with the values to inherit (template&modeClass)
      if(templateName == "prototype") {
        MigrationUtil.createArea(areasNode, "contentNavigation", "/templating-kit/pages/content/contentNavigation.ftl",  AreaDefinition.TYPE_NO_COMPONENT, false, null, "info.magnolia.module.templatingkit.templates.areas.ContentNavigationAreaModel", null, null, null)
      }

      //Correcting wrong template category
      setCorrectFeatureCategory(templateNode)

      //Adding the area to all content template definition in an enabled state
      if(templateNode.hasNodeData("category") && templateNode.getNodeData("category").getString()=="content"){
        MigrationUtil.createArea(areasNode, "contentNavigation", null,  AreaDefinition.TYPE_NO_COMPONENT, true, null, null, null, null, null)
      }
    }
  }

  /**
   * Correcting wrong template category.
   */
  private setCorrectFeatureCategory(Content templateNode) {
    def templatesContainingWrongCategory = [
      "stkFAQ",
      "stkImageGallery",
      "stkPublicUserRegistration"
    ]
    if(templatesContainingWrongCategory.contains(templateNode.getName()) && templateNode.hasNodeData("category")){
      templateNode.setNodeData("category", "feature")
    }
  }

  /**
   * This method transforms the extras area into extras sub areas. The amount of created extras sub areas is dependent on the columns value within the extras area itself.
   * Such as extras1 & extras2 when having configured columns=2
   * If no columns property id defined, no sub extras areas are created. We assume that everything is inherited by the prototype, including its extras sub areas.
   * The method expects the area node to be already transformed with the standard area transformations.
   * @param extrasAreaNode the node of the extras area
   */
  def Content transformExtrasToSubAreas(Content extrasAreaNode){
    if(extrasAreaNode.hasContent("availableComponents") || extrasAreaNode.hasContent("autoGeneration") || extrasAreaNode.hasNodeData("columns")){
      def columns
      if(extrasAreaNode.hasNodeData("columns")) {
        columns = extrasAreaNode.getNodeData("columns").getLong()
      } else {
        columns = 2;
      }
      def areasNode = MigrationUtil.createAreasNode(extrasAreaNode)

      def availableComponentsNode
      if(extrasAreaNode.hasContent("availableComponents")){
        availableComponentsNode = extrasAreaNode.getContent("availableComponents")
      }
      def autoGenerationNode
      if(extrasAreaNode.hasContent("autoGeneration")){
        autoGenerationNode = extrasAreaNode.getContent("autoGeneration")
      }
      def inheritanceNode
      if(extrasAreaNode.hasContent("inheritance")){
        inheritanceNode = extrasAreaNode.getContent("inheritance")
      }

      def amountExtras = 2
      if(columns > 2) amountExtras = columns
      def i=1
      while(i <= amountExtras){

        Content createdArea = MigrationUtil.createArea(areasNode, "extras"+i, "/templating-kit/pages/global/extrasSubArea.ftl",  AreaDefinition.TYPE_LIST, true, null, null, null, null, null)
        if(availableComponentsNode != null){
          ContentUtil.copyInSession(availableComponentsNode, createdArea.getHandle()+"/availableComponents")
        }
        if(autoGenerationNode != null){
          ContentUtil.copyInSession(autoGenerationNode, createdArea.getHandle()+"/autoGeneration")
        }
        if(inheritanceNode != null){
          ContentUtil.copyInSession(inheritanceNode, createdArea.getHandle()+"/inheritance")
        }
        i += 1
      }
      if(availableComponentsNode != null) availableComponentsNode.delete()
      if(autoGenerationNode != null) autoGenerationNode.delete()
      if(inheritanceNode != null) inheritanceNode.delete()


      hm.save()
    }

    return extrasAreaNode
  }

  /**
   * Transforms the footer area after standard area transformation.
   * Removes editable within the prototype. Removes the area everywhere else.
   * This because in Area inheritance the elements are not editable by default
   * @param footerArea the footer area
   */
  def Content specialTransformFooterArea(Content footerArea){
    if(MigrationUtil.isWithinPrototype(footerArea)) {
      footerArea.deleteNodeData("editable")
      return footerArea
    } else{
      footerArea.delete()
      return null
    }
  }

  /**
   * Comments area need transformation. Its paragraph property is obsolete and is added to the list of available components.
   *
   * @param commentsArea
   * @return
   */
  def transformCommentsArea(Content commentsArea){
    if(commentsArea.hasNodeData("paragraph")){
      def paragraph = NodeDataUtil.getString(commentsArea, "paragraph")
      MigrationUtil.createAvailableComponents(commentsArea, [paragraph], componentsIdMap)
      commentsArea.delete("paragraph")
    }
    MigrationUtil.addGenericTemplateScript(commentsArea,"single")
  }
}
