package scripts.migration.from4_4To4_5.stk;

import java.util.Collection;
import java.util.Map;

import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.cms.core.ItemType;
import info.magnolia.cms.util.ContentUtil;
import info.magnolia.context.Context;

import org.apache.commons.lang.StringUtils;

import scripts.migration.from4_4To4_5.util.MigrationUtil;

/**
 * This class transform the content in order that it reflects the new definition.
 * For example, in the defintion the opener area is within the main area.
 * It must be the same for the content, the opener node must be within the area node.
 *
 */
class TransformSTKWebsites {

  Context ctx
  HierarchyManager hm

  /**
   * Move intro / opener nodes to main node.
   *
   * @param page Page mode to transform.
   */
  def private transformMainNode(Content page){
    if(page.hasContent("main")){
      //Moving all content from main collection into sub area 'content'
      Content content = MigrationUtil.renameNode(page.getContent("main"), "content")
    }
  }

  /**
   * All the transformations to apply on a page.
   */
  def private transformContentOfPage(Content page){
    transformMainNode(page)
  }

  /**
   * Transform the content of a website.
   *
   * @param websiteName Site in website workspace to update. Name of site's root node name is expected.
   */
  def doWebsite(String websiteName){
    hm = ctx.getHierarchyManager("website")
    def website = hm.getContent(websiteName)
    transformContentOfPage(website)
    // visit all pages of the website
    ContentUtil.collectAllChildren(website, ItemType.CONTENT ).each{ Content page -> transformContentOfPage(page) }
    hm.save()
  }

  /**
   * Transform the content of a list of a website
   *
   * @param websitesList
   * @return
   */
  def doWebsites(Collection websitesList){
    websitesList.each{String siteName -> doWebsite(siteName) }
  }


}

