package scripts.migration.from4_4To4_5.util;

import java.lang.reflect.Array;

import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.cms.core.ItemType;
import info.magnolia.cms.core.NodeData;
import info.magnolia.cms.util.ContentUtil
import info.magnolia.cms.util.NodeDataUtil;
import info.magnolia.context.Context;
import info.magnolia.context.MgnlContext;
import info.magnolia.jcr.util.NodeUtil;

import org.apache.commons.lang.StringUtils;

class MigrationUtil {

  def static removeSTKPrefix(Content content) {
    def newName = StringUtils.uncapitalize(StringUtils.substringAfter(content.getName(), "stk"))
    ContentUtil.rename(content,newName)
    content.save()
  }

  /**
   * Build a map containing as key: the templateName, as value: the new id (moduleName:path/tothe/template)
   * Use this method to create the template id of a none-STK module
   *
   */
  def static Map buildTemplateIdMap(String moduleName, Context ctx) {

    def hm = ctx.getHierarchyManager('config')
    def module = hm.getContent("/modules/"+moduleName)
    def templateFolder = module.getContent("templates")
    def templatesList = []
    def templatesMap = [:]
    visitTemplates (templateFolder, templatesList)
    templatesList.each{ Content template ->
      templatesMap.put(template.getName(), createNewTemplateId(moduleName, template.getHandle()))
    }
    return templatesMap
  }

  /**
   * Build a map containing the pages id, as key: the templateName, as value: the new id (moduleName:path/tothe/template)
   * Can be call only after the 'Move Template' step
   * Use this method to create the template id of a STK-based module
   *
   */
  def static Map buildPagesIdMap(String moduleName, Context ctx) {
    return buildIdMap(moduleName, ctx, "pages")
  }

  def static buildPagesIdMapFromListOfModule(Collection moduleNameList, Context ctx) {
    def map = [:]
    moduleNameList.each{ String moduleName ->
      map.putAll(buildIdMap(moduleName, ctx, "pages"))
    }
    return map;
  }


  /**
   * Build a map containing the components id, as key: the templateName, as value: the new id (moduleName:path/tothe/template)
   * Can be call only after the 'Move Template' step
   * Use this method to create the template id of a STK-based module
   *
   */
  def static Map buildComponentsIdMap(String moduleName, Context ctx) {
    return buildIdMap(moduleName,ctx,"components")
  }

  def static Map buildComponentsIdMapFromListOfModule(Collection moduleNameList, Context ctx) {
    def map = [:]
    moduleNameList.each{ String moduleName ->
      map.putAll(buildIdMap(moduleName, ctx, "components"))
    }
    return map
  }

  def private static Map buildIdMap(String moduleName, Context ctx, String folderName) {
    def templatesList = []
    def templatesMap = [:]
    def hm = ctx.getHierarchyManager('config')
    def module = hm.getContent("/modules/"+moduleName+"/templates")
    if(module.hasContent(folderName)){
      def folder = module.getContent(folderName)
      visitTemplates (folder, templatesList)
      templatesList.each{ Content template ->
        templatesMap.put(template.getName(), createNewTemplateId(moduleName,template.getHandle()))
      }
    }
    return templatesMap
  }

  /**
   * Refactor the Template folder
   *
   */
  def static updateTemplatePropertiesGeneric(Content content, Map dialogPathMap, Collection dialogNotChanged) {
    def propertiesToChange =[:];
    propertiesToChange.put("template", "templateScript");
    propertiesToChange.put("type", "renderType");
    propertiesToChange.put("templatePath", "templateScript");

    propertiesToChange.each{ String oldPropName, String newPropName ->
      if(content.getNodeData(oldPropName).isExist())
        renameProperty(content.getNodeData(oldPropName), newPropName);
    }

    //update Dialog
    if(content.hasNodeData("dialog")){
      if(dialogPathMap.containsKey(content.getNodeData("dialog").getString())){
        content.setNodeData("dialog", dialogPathMap[content.getNodeData("dialog").getString()])

      }
      else{
        dialogNotChanged.add("Dialog not changed -> "+content.getNodeData("dialog").getString())
      }
      content.save()
    }
  }

  /**
   * Rename a property
   *
   */
  def static NodeData renameProperty(NodeData property, String newProperty)
  {
    //FIXME: Shouldn't that be 'property.value' otherwise always creating a String representation?
    def originalReference = property.value.string
    Content parentNode = property.parent
    // duplicate node data under new name 'extends'
    parentNode.setNodeData(newProperty, originalReference)
    // delete the old 'reference' node data
    property.delete()
    parentNode.save()
    return parentNode.getNodeData(newProperty)
  }

  /**
   * Rename a node.
   */
  def static Content renameNode(Content node, String newName) {
    ContentUtil.rename(node, newName)
    node.parent.save()
    return node
  }

  /**
  * Rename a node if it's exisiting - else do nothing and return null.
  */
 def static Content renameNodeIfExisting(HierarchyManager hm, String pathToNode, String newName) {
   if (!hm.isExist(pathToNode)) {
       return null;
   }
   def node = hm.getContent(pathToNode);
   ContentUtil.rename(node, newName)
   node.parent.save()
   return node
 }


  /**
   * The new id (moduleName:path/tothe/template)
   *
   */
  def static String createNewTemplateId(String moduleName, String templateHandle){
    return moduleName+":"+StringUtils.substringAfter(templateHandle,"templates/")
  }

  /**
   * Fill a list containing all the templates found (recursively) in the folder
   *
   */
  def static visitTemplates(Content folder, Collection list){
    ContentUtil.getAllChildren(folder).each{ Content child ->
      if(child.getItemType() == ItemType.CONTENTNODE){
        list.add(child)
      }
      if(child.getItemType() == ItemType.CONTENT){
        visitTemplates(child, list)
      }
    }
  }

  /**
   * Within a page, move a list of nodes (nodesToMove) into a node (nodeName)
   *
   */
  def static transformNode(Content page, String nodeName, Collection nodesPathsToMove){
    if(page.hasContent(nodeName)){
      def target = page.getContent(nodeName).getHandle()
      nodesPathsToMove.each{ String path ->
        if(page.hasContent(path)){
          def toMove = page.getContent(path)
          ContentUtil.moveInSession(toMove, target+"/"+path)
        }
      }
      page.save()
    }
  }

  /**
   *
   * @param parentAreasNode the 'areas' node to create the new area in
   * @param areaName the areas name
   * @param templateScript the template sript's path to set as property 'templateScript'
   * @param type the area type
   * @param enabled defines if the area should be enabled
   * @param beanClassName the bean classe's name representing the this area. String of class created as property 'class'
   * @param modelClassName the areas model classe's name. String is set as property 'modelClass'
   * @param availableComponentsList list of all available components to create in node 'availableComponents'
   * @return the created area node
   */
  def static Content createArea(Content parentAreasNode, String areaName, String templateScript, String type, boolean enabled, String beanClassName, String modelClassName, Collection availableComponentsList, Map parametersMap, Map componentsIdMap){
    def area = parentAreasNode.createContent(areaName, ItemType.CONTENTNODE);

    if(templateScript != null && StringUtils.isNotEmpty(templateScript)){
      area.setNodeData("templateScript", templateScript)
    }
    if(type != null && StringUtils.isNotEmpty(type)){
      area.setNodeData("type", type)
    }
    if(enabled == null && StringUtils.isEmpty(enabled)) enabled = false
    area.setNodeData("enabled", enabled)

    if(beanClassName != null && StringUtils.isNotEmpty(beanClassName)){
      area.setNodeData("class", beanClassName)
    }
    if(modelClassName != null && StringUtils.isNotEmpty(modelClassName)){
      area.setNodeData("modelClass", modelClassName)
    }
    if(availableComponentsList != null && availableComponentsList.size()>0 && componentsIdMap != null && componentsIdMap.iterator().hasNext()){
      createAvailableComponents(area, availableComponentsList, componentsIdMap)
    }
    if(parametersMap != null && parametersMap.iterator().hasNext()){
      createParameters(area, parametersMap)
    }
    return area;
  }

  /**
   * Create parameters for an Area.
   *
   * @param area The area node to create the parameters within.
   * @param parametersMap containing as key the label of the property and as value the value of the property
   */
  def static Content createParameters(Content area, Map parametersMap){
    if(parametersMap!=null && parametersMap.size()>0){
      if (!area.hasContent("parameters")) {
        area.createContent("parameters", ItemType.CONTENTNODE)
      }
      Content parameters = area.getContent("parameters")
      parametersMap.each{ String paraName, String paraValue ->
        parameters.setNodeData(paraName, paraValue)
      }
      return parameters
    }
  }

  def static Content createAvailableComponents(Content area, Collection availableComponentsList, Map componentIdMap)
  {
    if(availableComponentsList!=null && availableComponentsList.size() >0){
      //create availableComponents content node if it doesn't exist already
      if (!area.hasContent("availableComponents")) {
        area.createContent("availableComponents", ItemType.CONTENTNODE)
      }
      Content availableComponents = area.getContent("availableComponents")
      // create available components
      availableComponentsList.each{ String componentName ->
        if(componentIdMap.containsKey(componentName)){
          def id = componentIdMap.get(componentName)
          //create component
          def component = availableComponents.createContent(componentName, ItemType.CONTENTNODE)
          //set its id
          component.setNodeData("id", id)
        }
      }
      return availableComponents
    }
  }

  /**
   * Create a area node from the value found in the map areaObject.
   * The map must respect this structure
   *	areaObject = [
   *	    "areaName":"thename",
   *	    "type":"list",
   *	    "enabled": true,
   *	    "beanClassName":"beanClassName",
   *	    "modelClassName":"modelClassName"
   *	    "templateScript":"templateScript",
   *	    "availableComponents":["component1", "component2"],
   *	    "parameters":["label":"value","label2":"value2"]
   *	]
   *
   */
  def static Content createArea(Content parentAreasNode, Map areaObject, Map componentIdMap){
    //Ensouring that wrong passed types within the areaObject are casted if possible.
    String areaName = areaObject["areaName"]
    String templateScript = areaObject["templateScript"]
    String type = areaObject["type"]
    boolean enabled = areaObject["enabled"]
    String beanClassName = areaObject["beanClassName"]
    String modelClassName = areaObject["modelClassName"]
    Collection availableComponents = areaObject["availableComponents"]
    Map parameters = areaObject["parameters"]
    return createArea(parentAreasNode, areaName, templateScript, type, enabled, beanClassName, modelClassName, availableComponents, parameters, componentIdMap)
  }

  /**
   * Add a generic template script to an area
   *
   */
  def static addGenericTemplateScript(Content area, String type){
      // no such scripts needed because the areas render their subcomponents
  }

  /**
   * Return a generic template script
   *
   */
  def static String getGenericTemplateScript(String type){
      // no such scripts needed because the areas render their subcomponents
      return null
  }

  /**
   * Checks if the template is within the prototype
   * @param template the template to check
   * @return true if its within the prototype
   */
  def static boolean isWithinPrototype(Content template){
    def path = template.getHandle()
    return StringUtils.contains(path, "prototype")
  }

  /**
   * Updates the 'templateScript' property within the passed template definition.
   * @param template The template definition to update.
   * @param mapOfTemplateScriptsToChange a map of the template scripts to update containing the old(key) and the new value(value) as String.
   */
  def static updateChangedTemplateScripts(Content template, Map mapOfTemplateScriptsToChange){
    mapOfTemplateScriptsToChange.each{ String oldScriptPath, String newScriptPath ->
      updateTemplateScriptProperty(template, oldScriptPath, newScriptPath)
    }
  }

  /**
   * Updates the base path of the 'templateScript' property within the passed template definition.
   * 1. "/templating-kit/paragraphs/" are changed to "/templating-kit/components/".
   * 2. "/templating-kit/templates/" are changed to "/templating-kit/pages/".
   * @param template The template definition to update.
   * @param mapOfTemplateScriptsToChange a map of the template scripts to update containing the old(key) and the new value(value) as String.
   */
  def static updateBasePathInTemplateScriptProperty(Content template){
    replaceSubstringInProperty(template, "templateScript", "/templating-kit/paragraphs/", "/templating-kit/components/")
    replaceSubstringInProperty(template, "templateScript", "/templating-kit/templates/" , "/templating-kit/pages/")
    replaceSubstringInProperty(template, "templateScript", "/form/paragraphs/", "/form/components/")
  }

  /**
   * Updates the 'class' and 'modelClass' property within the passed template definition.
   * @param template The template definition to update.
   * @param mapOfClassesToChange a map of the classes to update containing the old(key) and the new value(value) as String.
   */
  def static updateClassAndModelClass(Content template, Map mapOfClassesToChange){
    //FIXME SCRUM-433 resolved - why iterate of a map when we have the key?
    mapOfClassesToChange.each{ String oldClassFQN, String newClassFQN ->
      updateClassProperty(template, oldClassFQN, newClassFQN)
      updateModelClassProperty(template, oldClassFQN, newClassFQN)
    }
  }

  /**
   * Updates the 'templateScript' property within the passed template definition.
   * @param template The template definition to update.
   * @param oldValue The value to update.
   * @param newValue The new value to set.
   */
  def static updateTemplateScriptProperty(Content template, String  oldValue, String newValue){
    if(template.hasNodeData("templateScript") && oldValue.equals(template.getNodeData("templateScript").getString())){
      template.setNodeData("templateScript", newValue)
    }
  }

  /**
   * Updates the 'class' property within the passed template definition.
   * @param template The template definition to update.
   * @param oldValue The value to update.
   * @param newValue The new value to set.
   */
  def static updateClassProperty(Content template, String oldValue, String newValue){
    if(template.hasNodeData("class") && oldValue.equals(template.getNodeData("class").getString())){
      template.setNodeData("class", newValue)
    }
  }

  /**
   * Updates the 'modelClass' property within the passed template definition.
   * @param template The template definition to update.
   * @param oldValue The value to update.
   * @param newValue The new value to set.
   */
  def static updateModelClassProperty(Content template, String oldValue, String newValue){
    if(template.hasNodeData("modelClass") && oldValue.equals(template.getNodeData("modelClass").getString())){
      template.setNodeData("modelClass", newValue)
    }
  }

  /**
   * Replaces the 'toReplace' value by the 'replaceWith' value in the 'propertyName' property within the passed template definition.
   * @param template The template definition to update.
   * @param propertyName the propery where to replace the substrings
   * @param toReplace substring to replace
   * @param replaceWith replace substring with
   */
  def static replaceSubstringInProperty(Content node, String propertyName, String toReplace, String replaceWith){
    if(node.hasNodeData(propertyName) ){
      def path = node.getNodeData(propertyName).getString()
      if(StringUtils.contains(path, toReplace)){
        path = StringUtils.replace(path, toReplace, replaceWith)
        node.setNodeData(propertyName, path)
      }
    }
  }

  /**
   * A map of all the freemeraker scripts which were moved and/or renamed within the STK. There for the fqn of the scripts changed.
   * This map provides information to update also projects with these script name changes.
   * @return A map of all the freemarker scripts which were moved and/or renamed. The key is the old script path value, the value is the new script path value.
   */
  def static Map mapOfChangedFreemarkerScripts(){
    def changedScripts = [
        //STK component scripts
        "/templating-kit/paragraphs/teasers/stkTeaserCombinedFeed.ftl" : "/templating-kit/components/teasers/combinedFeed.ftl",
        "/templating-kit/paragraphs/teasers/stkTeaserSingleFeed.ftl"   : "/templating-kit/components/teasers/singleFeed.ftl",
        "/templating-kit/paragraphs/pur/stkPURAuthSubmit.ftl"          : "/templating-kit/components/pur/authSubmit.ftl",
        "/templating-kit/paragraphs/pur/stkPURIntranet.ftl"            : "/templating-kit/components/pur/intranet.ftl",
        "/templating-kit/paragraphs/pur/stkPURLogin.ftl"               : "/templating-kit/components/pur/login.ftl",
        "/templating-kit/paragraphs/pur/stkPURNotAuthSubmit.ftl"       : "/templating-kit/components/pur/notAuthSubmit.ftl",
        "/templating-kit/paragraphs/content/DownloadList.ftl"          : "/templating-kit/components/content/downloadList.ftl",
        //STK page scripts
        "templating-kit/templates/functional/redirect.ftl"             : "/templating-kit/pages/functional/redirectMain.ftl", //the missing '/' on the key is on purpose, was a bug on old STK.
        "/templating-kit/templates/singleton/mainArea.ftl"             : "/templating-kit/pages/global/mainArea.ftl"
    ]
    return changedScripts
  }

  /**
   * A map of all the classes which were moved within the STK. There for the fqn of the classes changed.
   * This map provides information to update also projects with these class changes.
   * @return A map of all the classes which were moved. The key is the old class value, the value is the new class value.
   *
   * FIXME SCRUM-433 resolved: is it the right place for that, not really a generic util!! Ever project using these classes will need to update.
   */
  def static Map mapOfMovedClasses(){
    def movedClasses = [
          //STK moved classes
          "info.magnolia.module.templatingkit.paragraphs.AbstractItemListModel"		    : "info.magnolia.module.templatingkit.templates.components.AbstractItemListModel",
          "info.magnolia.module.templatingkit.paragraphs.AbstractDateContentModel"	    : "info.magnolia.module.templatingkit.templates.components.AbstractDateContentModel",
          "info.magnolia.module.templatingkit.paragraphs.EventsListModel"				: "info.magnolia.module.templatingkit.templates.components.EventsListModel",
          "info.magnolia.module.templatingkit.paragraphs.CalendarModel"				    : "info.magnolia.module.templatingkit.templates.components.CalendarModel",
          "info.magnolia.module.templatingkit.paragraphs.EventsOverviewModel"			: "info.magnolia.module.templatingkit.templates.components.EventsOverviewModel",
          "info.magnolia.module.templatingkit.paragraphs.NewsListModel"				    : "info.magnolia.module.templatingkit.templates.components.NewsListModel",
          "info.magnolia.module.templatingkit.paragraphs.NewsOverviewModel"			    : "info.magnolia.module.templatingkit.templates.components.NewsOverviewModel",
          "info.magnolia.module.templatingkit.paragraphs.CarouselModel"				    : "info.magnolia.module.templatingkit.templates.components.CarouselModel",
          "info.magnolia.module.templatingkit.paragraphs.ContactModel"				    : "info.magnolia.module.templatingkit.templates.components.ContactModel",
          "info.magnolia.module.templatingkit.paragraphs.ContentTypeSyndicateModel"	    : "info.magnolia.module.templatingkit.templates.components.ContentTypeSyndicateModel",
          "info.magnolia.module.templatingkit.paragraphs.DownloadLinkModel"			    : "info.magnolia.module.templatingkit.templates.components.DownloadLinkModel",
          "info.magnolia.module.templatingkit.paragraphs.DownloadListModel"			    : "info.magnolia.module.templatingkit.templates.components.DownloadListModel",
          "info.magnolia.module.templatingkit.paragraphs.ExternalLinkModel"			    : "info.magnolia.module.templatingkit.templates.components.ExternalLinkModel",
          "info.magnolia.module.templatingkit.paragraphs.ExternalTeaserModel"			: "info.magnolia.module.templatingkit.templates.components.ExternalTeaserModel",
          "info.magnolia.module.templatingkit.paragraphs.FlashModel"					: "info.magnolia.module.templatingkit.templates.components.FlashModel",
          "info.magnolia.module.templatingkit.paragraphs.ImageGalleryParagraphModel"	: "info.magnolia.module.templatingkit.templates.components.ImageGalleryParagraphModel",
          "info.magnolia.module.templatingkit.paragraphs.InternalTeaserModel"			: "info.magnolia.module.templatingkit.templates.components.InternalTeaserModel",
          "info.magnolia.module.templatingkit.paragraphs.DownloadTeaserModel"			: "info.magnolia.module.templatingkit.templates.components.DownloadTeaserModel",
          "info.magnolia.module.templatingkit.paragraphs.AbstractTeaserModel"			: "info.magnolia.module.templatingkit.templates.components.AbstractTeaserModel",
          "info.magnolia.module.templatingkit.paragraphs.FooterImageModel"			    : "info.magnolia.module.templatingkit.templates.components.FooterImageModel",
          "info.magnolia.module.templatingkit.paragraphs.TextImageModel"				: "info.magnolia.module.templatingkit.templates.components.TextImageModel",
          "info.magnolia.module.templatingkit.paragraphs.ImageModel"					: "info.magnolia.module.templatingkit.templates.components.ImageModel",
          "info.magnolia.module.templatingkit.paragraphs.InternalLinkModel"			    : "info.magnolia.module.templatingkit.templates.components.InternalLinkModel",
          "info.magnolia.module.templatingkit.paragraphs.SiteMapParagraphModel"		    : "info.magnolia.module.templatingkit.templates.components.SiteMapParagraphModel",
          "info.magnolia.module.templatingkit.paragraphs.FlashParagraph"				: "info.magnolia.module.templatingkit.templates.components.FlashParagraph",
          "info.magnolia.module.templatingkit.paragraphs.StageTeaserModel"			    : "info.magnolia.module.templatingkit.templates.components.StageTeaserModel",
          "info.magnolia.module.templatingkit.paragraphs.TabbedTeaserModel"			    : "info.magnolia.module.templatingkit.templates.components.TabbedTeaserModel",
          "info.magnolia.module.templatingkit.templates.STKTemplate"					: "info.magnolia.module.templatingkit.templates.pages.STKPage",
          //FORM moved classes
          "info.magnolia.module.form.paragraphs.models.AbstractFormEngine"				: "info.magnolia.module.form.templates.components.AbstractFormEngine",
          "info.magnolia.module.form.paragraphs.models.AbstractFormModel"				: "info.magnolia.module.form.templates.components.AbstractFormModel",
          "info.magnolia.module.form.paragraphs.models.DefaultFormDataBinder"			: "info.magnolia.module.form.templates.components.DefaultFormDataBinder",
          "info.magnolia.module.form.paragraphs.models.ErrorView"				        : "info.magnolia.module.form.templates.components.ErrorView",
          "info.magnolia.module.form.paragraphs.models.FormFieldModel"			        : "info.magnolia.module.form.templates.components.FormFieldModel",
          "info.magnolia.module.form.paragraphs.models.FormModel"				        : "info.magnolia.module.form.templates.components.FormModel",
          "info.magnolia.module.form.paragraphs.models.FormSummaryBean"			    	: "info.magnolia.module.form.templates.components.FormSummaryBean",
          "info.magnolia.module.form.paragraphs.models.FormSummaryModel"				: "info.magnolia.module.form.templates.components.FormSummaryModel",
          "info.magnolia.module.form.paragraphs.models.FormView"				        : "info.magnolia.module.form.templates.components.FormView",
          "info.magnolia.module.form.paragraphs.models.SessionExpiredView"				: "info.magnolia.module.form.templates.components.SessionExpiredView",
          "info.magnolia.module.form.paragraphs.models.SubStepFormModel"				: "info.magnolia.module.form.templates.components.SubStepFormModel",
          "info.magnolia.module.form.paragraphs.models.SuccessView"			        	: "info.magnolia.module.form.templates.components.SuccessView",
          "info.magnolia.module.form.templates.FormParagraph"			            	: "info.magnolia.module.form.templates.components.FormParagraph",
          "info.magnolia.module.form.templates.FormStepParagraph"		        		: "info.magnolia.module.form.templates.components.FormStepParagraph",
          "info.magnolia.module.form.templates.ParagraphConfig"			            	: "info.magnolia.module.form.templates.components.ParagraphConfig",
          "info.magnolia.module.form.paragraphs.models.multistep.GoToFirstPageView"		: "info.magnolia.module.form.templates.components.ParagraphConfig",
          "info.magnolia.module.form.paragraphs.models.multistep.NavigationUtils"		: "info.magnolia.module.form.templates.components.NavigationUtils",
          "info.magnolia.module.form.paragraphs.models.multistep.StartStepFormEngine"	: "info.magnolia.module.form.templates.components.StartStepFormEngine",
          "info.magnolia.module.form.paragraphs.models.multistep.SubStepFormEngine"		: "info.magnolia.module.form.templates.components.SubStepFormEngine",
          "info.magnolia.module.form.paragraphs.models.FormFieldModel"					: "info.magnolia.module.form.templates.components.FormFieldModel",
          "info.magnolia.module.form.paragraphs.models.SubStepFormModel"				: "info.magnolia.module.form.templates.components.SubStepFormModel",
          "info.magnolia.module.form.paragraphs.models.FormSummaryModel"				: "info.magnolia.module.form.templates.components.FormSummaryModel",
          //RSS moved class
          "info.magnolia.module.rssaggregator.paragraphs.AbstractFeedModel"				: "info.magnolia.module.rssaggregator.templates.components.AbstractFeedModel",
          "info.magnolia.module.rssaggregator.paragraphs.FeedListModel"					: "info.magnolia.module.rssaggregator.templates.components.FeedListModel",
          "info.magnolia.module.rssaggregator.paragraphs.CombinedFeedModel"				: "info.magnolia.module.rssaggregator.templates.components.CombinedFeedModel"
        ]
    return movedClasses
  }

  /**
   * A map of all the classes which were deprecated and have a replacement class within the STK.
   * There for these deprecated classes need to be updated.
   * This map provides information to update also projects with these class changes.
   * @return A map of all the classes which were moved. The key is the old class value, the value is the new class value.
   */
  def static Map mapOfDeprecatedClassesToReplace(){
    def deprecatedClassesToUpdate = [
          //STK deprecated classes to replace
          "info.magnolia.module.templatingkit.templates.STKTemplateModel"                 : "info.magnolia.module.templatingkit.templates.pages.STKPageModel",
          "info.magnolia.module.templatingkit.templates.SingletonParagraphTemplateModel"  : "info.magnolia.module.templatingkit.templates.pages.STKPageModel",
          "info.magnolia.module.templatingkit.templates.SingletonParagraphMainArea"       : "info.magnolia.module.templatingkit.templates.MainArea"
        ]
    return deprecatedClassesToUpdate
  }

  /**
   * The singleton/mainArea script does not exist anymore. Its already changes by call on mapOfChangedFreemarkerScripts().
   * This map provides information about the changes of the mainArea scripts:
   * - The prototype & all singletons use now the general/mainArea.ftl and not anymore the content/mainArea.ftl.
   * - The content/mainArea.ftl is now only used explicit by the content templates.
   *
   * @return Map of the changed mainArea scripts. key -> template name, value -> new script
   */
  def static Map mapOfChangedMainAreaScriptsByTemplate(){
      Map mainAreaScriptsForTemplates = [
          //Site Definition's prototype
          "prototype"       :"/templating-kit/pages/global/mainArea.ftl",
          //Content templates
          "stkArticle"      : "/templating-kit/pages/content/mainArea.ftl",
          "stkArticleLarge" : "/templating-kit/pages/content/mainArea.ftl",
          "stkNews"         : "/templating-kit/pages/content/mainArea.ftl",
          "stkEvents"       : "/templating-kit/pages/content/mainArea.ftl",
          "stkGlossaryTerm" : "/templating-kit/pages/content/mainArea.ftl",
          ]
      return mainAreaScriptsForTemplates
  }

  /**
   * Creates a master (for dialogs, paragraphs, templates,..) as copy from an existing node.
   * Optionally, subnodes can be either removed or renamed.
   *
   * @param ctx
   * @param originalPath Path to the dialog, paragraph, template that provides the base for the master.
   * @param masterPath Path to where the new master should be copied. Parent path MUST exist.
   * @param optionsMapWithRelativePaths A map containing relative paths (to masterPath) of nodes that either need to be deleted or renamed. Pattern: [path : newName or blank for deletion]
   *
   */
  def static Content createMasterDialogThroughCopy(Context ctx, String originalPath, String masterPath, Map optionsMapWithRelativePaths){
    HierarchyManager hm = ctx.getHierarchyManager("config")
    Content originalNode = hm.getContent(originalPath)

    ContentUtil.copyInSession(originalNode, masterPath)

    Content masterNode = hm.getContent(masterPath)
    masterNode.parent.save()

    if (!optionsMapWithRelativePaths.isEmpty()) {
      optionsMapWithRelativePaths.each{ String childNodeRelativePath, String optionValue ->
        def combinedPath = masterPath + childNodeRelativePath
        if (hm.isExist(combinedPath)) {
            def childNode = hm.getContent(combinedPath)
            if (StringUtils.isNotBlank(optionValue)) {
                renameNode(childNode, optionValue)
            }
            else {
                childNode.delete()
            }
        }
      }
    }
    //save changes on child nodes
    masterNode.save()
    return masterNode
  }

  /**
   * Create a node 'areas' within the passed content node.
   * If an area node already exists, the existing 'areas' node is returned.
   *
   * @param content The node to create an 'areas' node within.
   * @return the areas node which was created or found.
   */
  def static Content createAreasNode(Content content){
    if(!content.hasContent("areas")){
      return content.createContent("areas", ItemType.CONTENTNODE);
    }
    return content.getContent("areas");
  }

  /**
   * Create a node if not existing. If existing the existing node is returned.
   * @param parent Node to create the child node in.
   * @param name Name of the child node to create.
   * @param itemType Type of the node to create.
   * @return
   */
  def static Content getOrCreateContent(Content parent, String name, ItemType itemType){
    if(parent.hasContent(name)){
      return parent.getContent(name)
    }
    return parent.createContent(name, itemType);
  }

  /**
   * Removed from the passed list all modules which are not installed.
   * @param moduleNames A list of module to check if they are installed.
   */
  def static Collection filterUninstalledModules(Collection possibleModuleNames){
    Collection installedModules = []
    possibleModuleNames.each{ String moduleName ->
      if(moduleisInstalled(moduleName)){
        installedModules.add(moduleName)
      }
    }
    return installedModules
  }

  /**
   * Checks if the module is installed.
   * @param moduleName The module to check if it is installed
   */
  def static boolean moduleisInstalled(String moduleName){
    def hm = MgnlContext.getHierarchyManager("config")
    return hm.isExist("/modules/"+moduleName)
  }

  /**
   * Remove from the passed list all non existing web sites.
   * @param webSitePaths Paths of the web sites to check it they exist.
   */
  def static Collection filterNotExistingWebSites(Collection webSitePaths){
    Collection existingWebSites = []
    webSitePaths.each{ String webSitePath ->
      if(webSiteExists(webSitePath)){
        existingWebSites.add(webSitePath)
      }
    }
    return existingWebSites
  }

  /**
   * Checks if the web site exists.
   * @param webSitePath Path to the web site to check.
   */
  def static boolean webSiteExists(String webSitePath){
    def hm = MgnlContext.getHierarchyManager("website")
    return hm.isExist(webSitePath)
  }


}
